"""File:  CanvasTest_tags.py  Test animation with .update method to force
   the screen updates and time.sleep to delay between moves"""
from Tkinter import *
import time
import sys

class Application(Frame):
    SPEED = 5
    def __init__(self, master=None):
        Frame.__init__(self, master)
        self.master.title("Canvas and Mouse-events Example")
        
        self._dragging = False  # remembers if mouse button pressed
        self._currentShapeID = None
        self._shapeVar = StringVar()   # type of shape
        self._shapeVar.set("circle")   # initial shape
        self.grid(sticky=N+S+E+W)
        self._createWidgets()

    def _createWidgets(self):
        """Create GUI widgets"""
        self.shapeMenubutton = Menubutton(self, text="Shape", relief=RAISED)
        self.shapeMenubutton.grid(row=0, column=0, sticky=N)
        
        self.shapeMenubutton.menu = Menu(self.shapeMenubutton, tearoff=0)
        self.shapeMenubutton["menu"] = self.shapeMenubutton.menu
        self.shapeMenubutton.menu.add_radiobutton( label="oval", command=self._shapeSelection,
                                                   value = "circle", variable=self._shapeVar)
        self.shapeMenubutton.menu.add_radiobutton( label="rectangle",
                                                   command=self._shapeSelection,
                                                   value = "top_left_corner",
                                                   variable=self._shapeVar)

        self._myCanvas = Canvas(self, cursor=self._shapeVar.get() ,borderwidth=4,
                               height='5i', width='7i',scrollregion=(0,0,1000,3000))
        self._myCanvas.grid(row=1, column=0, columnspan=2)
        self._yScroll = Scrollbar(self, orient=VERTICAL, command=self._yScrollCanvas)
        self._yScroll.grid(row=1,column=2, sticky=N+S)
        self._myCanvas["yscrollcommand"]=self._yScroll.set

        self._myCanvas.bind("<Button-1>", self._startShape)
        self._myCanvas.bind("<Motion>", self._sizeShape)
        self._myCanvas.bind("<ButtonRelease-1>", self._endShape)
        
        self.doneButton = Button(self, text='Click when done', command=self._doneSelections)
        self.doneButton.grid(row=3, column=0, columnspan=2,sticky=W+E)
        self._myCanvas.bind("<Button-3>", self._moveTowardCursor)

    def _moveTowardCursor(self,event):
        print 'moving',self._shapeVar.get(),'toward cursor'
        self._moveShape(self._shapeVar.get(), event.x, event.y)

    def _moveShape(self, tag, cursor_x, cursor_y):
        cursor_x = float(cursor_x)
        cursor_y = float(cursor_y)
        print 'IN _moveShape: tag =',tag,'x=',cursor_x, 'y=', cursor_y
        idList = self._myCanvas.find_withtag(tag)
        for i in xrange(5):
            for shapeId in idList:
                coords = self._myCanvas.coords(shapeId)
                x0,y0,x1,y1 = coords
                print 'coords=',coords
                xDiff = (cursor_x-((x1+x0)/2))
                yDiff = (cursor_y-((y1+y0)/2))
                print 'shapeId =',shapeId,'xDiff=',xDiff, 'yDiff=',yDiff
                if xDiff > Application.SPEED:
                    xDiff = Application.SPEED
                elif xDiff < -Application.SPEED:
                    xDiff = -Application.SPEED
                else:
                    xDiff = 0

                if yDiff > Application.SPEED:
                    yDiff = Application.SPEED
                elif yDiff < -Application.SPEED:
                    yDiff = -Application.SPEED
                else:
                    yDiff = 0
                print 'shapeId =',shapeId,'xDiff=',xDiff, 'yDiff=',yDiff
                self._myCanvas.move(shapeId, xDiff, yDiff)
            self._myCanvas.update()
            time.sleep(0.1)


    def _yScrollCanvas(self, cmdType, direction, unit="units"):
        """Scrolls the Canvas vertically"""
        if cmdType == "scroll":
            self._myCanvas.yview_scroll(direction, unit)
        elif cmdType == "moveto":
            self._myCanvas.yview_moveto(direction)
        
    def _shapeSelection(self):
        """When shape radiobutton selected set the canvas cursor"""
        print "Shape selected:", self._shapeVar.get()
        self._myCanvas["cursor"] = self._shapeVar.get()

    def _startShape(self, event):
        """When left mouse button pressed, create the selected shape and set _dragging flag"""
        xCanvas = self._myCanvas.canvasx(event.x)
        yCanvas = self._myCanvas.canvasy(event.y)
        print "startShape:  shape =", self._shapeVar.get(),"at x =", xCanvas,"y =", yCanvas
        self._dragging = True
        if self._shapeVar.get() == "circle":
            self._currentShapeID = self._myCanvas.create_oval(xCanvas, yCanvas, xCanvas, yCanvas)
            self._myCanvas.itemconfig(self._currentShapeID, tag='circle',fill='red')    
        elif self._shapeVar.get() == "top_left_corner":
            self._currentShapeID = self._myCanvas.create_rectangle(xCanvas, yCanvas, xCanvas, yCanvas, tag='top_left_corner',fill='blue')
        
    def _sizeShape(self, event):
        """When mouse is moved and dragging is True, create a new shape of the correct size."""
        if self._dragging:
            oldCoords = self._myCanvas.coords(self._currentShapeID)
            xCanvas = self._myCanvas.canvasx(event.x)
            yCanvas = self._myCanvas.canvasy(event.y)
            self._myCanvas.delete(self._currentShapeID)
            if self._shapeVar.get() == "circle":
                self._currentShapeID = self._myCanvas.create_oval(oldCoords[0], oldCoords[1],
                                                                xCanvas, yCanvas)
                self._myCanvas.itemconfig(self._currentShapeID, tag='circle',fill='red')    
            elif self._shapeVar.get() == "top_left_corner":
                self._currentShapeID = self._myCanvas.create_rectangle(oldCoords[0], oldCoords[1],
                                                                     xCanvas, yCanvas, tag='top_left_corner',fill='blue')

    def _endShape(self, event):
        """When mouse button released, set _dragging flag to False"""
        self._dragging = False
    
    def _doneSelections(self):
        """When done button is clicked, exit and save drawing to encapsulated postscript file"""
        print "Done:  Drawing written to file 'SimpleDrawOut.eps'"
        self._myCanvas.postscript(file="SimpleDrawOut.eps")
        sys.exit

def main():
    """Instantiate and pop up the window."""
    Application().mainloop()

main()
